@@ -1,7 +1,7 @@
-#' Does code print output to the console?
+#' Do you expect printed output to match this pattern?
 #'
 #' Test for output produced by `print()` or `cat()`. This is best used for
-#' very simple output; for more complex cases use [verify_output()].
+#' very simple output; for more complex cases use [expect_snapshot()].
 #'
 #' @export
 #' @family expectations
@@ -22,31 +22,39 @@
 #' # You can use the arguments of grepl to control the matching
 #' expect_output(str(mtcars), "11 VARIABLES", ignore.case = TRUE)
 #' expect_output(str(mtcars), "$ mpg", fixed = TRUE)
-expect_output <- function(object,
-                          regexp = NULL,
-                          ...,
-                          info = NULL,
-                          label = NULL,
-                          width = 80
-                          ) {
+expect_output <- function(
+  object,
+  regexp = NULL,
+  ...,
+  info = NULL,
+  label = NULL,
+  width = 80
+) {
+  check_number_whole(width, min = 1)
+
   act <- quasi_capture(enquo(object), label, capture_output, width = width)
 
   if (identical(regexp, NA)) {
-    expect(
-      identical(act$cap, ""),
-      sprintf("%s produced output.\n%s", act$lab, encodeString(act$cap)),
-      info = info
-    )
+    if (!identical(act$cap, "")) {
+      msg <- c(
+        sprintf("Expected %s to produce no output.", act$lab),
+        sprintf("Actual output:\n%s", encodeString(act$cap))
+      )
+      fail(msg, info = info)
+    } else {
+      pass()
+    }
   } else if (is.null(regexp) || identical(act$cap, "")) {
-    expect(
-      !identical(act$cap, ""),
-      sprintf("%s produced no output", act$lab),
-      info = info
-    )
+    if (identical(act$cap, "")) {
+      msg <- sprintf("Expected %s to produce output.", act$lab)
+      fail(msg, info = info)
+    } else {
+      pass()
+    }
   } else {
-    expect_match(act$cap, enc2native(regexp), ..., info = info, label = act$lab)
+    act_out <- labelled_value(act$cap, paste0("output from ", act$lab))
+    expect_match_(act_out, enc2native(regexp), ..., title = "output")
   }
 
   invisible(act$val)
 }
-
