@@ -1,5 +1,4 @@
-
-# +-----------------------------+     +-------------------------------+
+# /-----------------------------\     /-------------------------------\
 # | Main R process              |     | Subprocess 1                  |
 # | +------------------------+  |     | +---------------------------+ |
 # | | test_dir_parallel()    |  |     | | test_file()               | |
@@ -12,17 +11,16 @@
 # | | | Progress2Reporter |  |  |  |  | | |    v                  | | |
 # | | +-------------------+  |  |  |  | | | +-------------------+ | | |
 # | +------------------------+  |  |--------| signalCondition() | | | |
-# +-----------------------------+  |  | | | +-------------------+ | | |
+# \-----------------------------/  |  | | | +-------------------+ | | |
 #                                  |  | | +-----------------------+ | |
 #                                  |  | +---------------------------+ |
-#                                  |  +-------------------------------+
-#                                  |  +-------------------------------+
+#                                  |  \-------------------------------/
+#                                  |  /-------------------------------\
 #                                  |--| Subprocess 2                  |
-#                                  |  +-------------------------------+
-#                                  |  +-------------------------------+
-#                                  +--| Subprocess 3                  |
-#                                     +-------------------------------+
-#                                       ...
+#                                  |  \-------------------------------/
+#                                  |  /-------------------------------\
+#                                  \--| Subprocess 3                  |
+#                                     \-------------------------------/
 #
 # ## Notes
 #
@@ -33,25 +31,22 @@
 #   runs an event loop.
 
 test_files_parallel <- function(
-                       test_dir,
-                       test_package,
-                       test_paths,
-                       load_helpers = TRUE,
-                       reporter = default_parallel_reporter(),
-                       env = NULL,
-                       stop_on_failure = FALSE,
-                       stop_on_warning = FALSE,
-                       wrap = TRUE,  # unused, to match test_files signature
-                       load_package = c("none", "installed", "source")
-                       ) {
-
+  test_dir,
+  test_package,
+  test_paths,
+  load_helpers = TRUE,
+  reporter = default_parallel_reporter(),
+  env = NULL,
+  stop_on_failure = FALSE,
+  stop_on_warning = FALSE,
+  wrap = TRUE, # unused, to match test_files signature
+  load_package = c("none", "installed", "source"),
+  shuffle = FALSE
+) {
   # TODO: support timeouts. 20-30s for each file by default?
 
   num_workers <- min(default_num_cpus(), length(test_paths))
-  inform(paste0(
-    "Starting ", num_workers, " test process",
-    if (num_workers != 1) "es"
-  ))
+  cli::cli_inform("Starting {num_workers} test process{?es}.")
 
   # Set up work queue ------------------------------------------
   queue <- NULL
@@ -64,11 +59,12 @@ test_files_parallel <- function(
     test_dir = test_dir,
     load_helpers = load_helpers,
     num_workers = num_workers,
-    load_package = load_package
+    load_package = load_package,
+    shuffle = shuffle
   )
 
   withr::with_dir(test_dir, {
-    reporters <- test_files_reporter_parallel(reporter)
+    reporters <- test_files_reporter(reporter, "parallel")
     with_reporter(reporters$multi, {
       parallel_updates <- reporter$capabilities$parallel_updates
       if (parallel_updates) {
@@ -78,37 +74,26 @@ test_files_parallel <- function(
       }
     })
 
-    test_files_check(reporters$list$get_results(),
+    test_files_check(
+      reporters$list$get_results(),
       stop_on_failure = stop_on_failure,
       stop_on_warning = stop_on_warning
     )
   })
 }
 
-test_files_reporter_parallel <- function(reporter, .env = parent.frame()) {
-  lister <- ListReporter$new()
-  snapshotter <- MainprocessSnapshotReporter$new("_snaps", fail_on_new = FALSE)
-  reporters <- list(
-    find_reporter(reporter),
-    lister, # track data
-    snapshotter
-  )
-  withr::local_options(
-    "testthat.snapshotter" = snapshotter,
-    .local_envir = .env
-  )
-  list(
-    multi = MultiReporter$new(reporters = compact(reporters)),
-    list = lister
-  )
-}
 
 default_num_cpus <- function() {
   # Use common option, if set
   ncpus <- getOption("Ncpus", NULL)
   if (!is.null(ncpus)) {
     ncpus <- suppressWarnings(as.integer(ncpus))
-    if (is.na(ncpus)) abort("`getOption(Ncpus)` must be an integer")
+    if (is.na(ncpus)) {
+      cli::cli_abort(
+        "{.code getOption('Ncpus')} must be an integer.",
+        call = NULL
+      )
+    }
     return(ncpus)
   }
 
@@ -116,7 +101,9 @@ default_num_cpus <- function() {
   ncpus <- Sys.getenv("TESTTHAT_CPUS", "")
   if (ncpus != "") {
     ncpus <- suppressWarnings(as.integer(ncpus))
-    if (is.na(ncpus)) abort("TESTTHAT_CPUS must be an integer")
+    if (is.na(ncpus)) {
+      cli::cli_abort("{.envvar TESTTHAT_CPUS} must be an integer.")
+    }
     return(ncpus)
   }
 
@@ -138,13 +125,19 @@ parallel_event_loop_smooth <- function(queue, reporters, test_dir) {
 
     updated <- FALSE
     for (x in msgs) {
+      if (x$code == PROCESS_OUTPUT) {
+        lns <- paste0("> ", x$path, ": ", x$message)
+        cat("\n", file = stdout())
+        base::writeLines(lns, stdout())
+        next
+      }
       if (x$code != PROCESS_MSG) {
         next
       }
 
       m <- x$message
       if (!inherits(m, "testthat_message")) {
-        message(m)
+        cli::cli_inform(as.character(m))
         next
       }
 
@@ -173,13 +166,18 @@ parallel_event_loop_chunky <- function(queue, reporters, test_dir) {
   while (!queue$is_idle()) {
     msgs <- queue$poll(Inf)
     for (x in msgs) {
+      if (x$code == PROCESS_OUTPUT) {
+        lns <- paste0("> ", x$path, ": ", x$message)
+        base::writeLines(lns, stdout())
+        next
+      }
       if (x$code != PROCESS_MSG) {
         next
       }
 
       m <- x$message
       if (!inherits(m, "testthat_message")) {
-        message(m)
+        cli::cli_inform(as.character(m))
         next
       }
 
@@ -207,16 +205,20 @@ replay_events <- function(reporter, events) {
   }
 }
 
-queue_setup <- function(test_paths,
-                        test_package,
-                        test_dir,
-                        num_workers,
-                        load_helpers,
-                        load_package) {
-
+queue_setup <- function(
+  test_paths,
+  test_package,
+  test_dir,
+  num_workers,
+  load_helpers,
+  load_package,
+  shuffle = FALSE
+) {
   # TODO: observe `load_package`, but the "none" default is not
   # OK for the subprocess, because it'll not have the tested package
-  if (load_package == "none") load_package <- "source"
+  if (load_package == "none") {
+    load_package <- "source"
+  }
 
   # TODO: similarly, load_helpers = FALSE, coming from devtools,
   # is not appropriate in the subprocess
@@ -228,7 +230,8 @@ queue_setup <- function(test_paths,
 
   # First we load the package "manually", in case it is testthat itself
   load_hook <- expr({
-    switch(!!load_package,
+    switch(
+      !!load_package,
       installed = library(!!test_package, character.only = TRUE),
       source = pkgload::load_all(!!test_dir, helpers = FALSE, quiet = TRUE)
     )
@@ -248,15 +251,22 @@ queue_setup <- function(test_paths,
   })
   queue <- task_q$new(concurrency = num_workers, load_hook = load_hook)
 
-  fun <- transport_fun(function(path) asNamespace("testthat")$queue_task(path))
+  fun <- transport_fun(function(path, shuffle) {
+    asNamespace("testthat")$queue_task(path, shuffle)
+  })
   for (path in test_paths) {
-    queue$push(fun, list(path))
+    queue$push(fun, list(path, shuffle))
   }
 
   queue
 }
 
-queue_process_setup <- function(test_package, test_dir, load_helpers, load_package) {
+queue_process_setup <- function(
+  test_package,
+  test_dir,
+  load_helpers,
+  load_package
+) {
   env <- asNamespace("testthat")$test_files_setup_env(
     test_package,
     test_dir,
@@ -276,19 +286,19 @@ queue_process_setup <- function(test_package, test_dir, load_helpers, load_packa
   the$testing_env <- env
 }
 
-queue_task <- function(path) {
+queue_task <- function(path, shuffle = FALSE) {
   withr::local_envvar("TESTTHAT_IS_PARALLEL" = "true")
-  snapshotter <- SubprocessSnapshotReporter$new(
-    snap_dir = "_snaps",
-    fail_on_new = FALSE
-  )
+  snapshotter <- SubprocessSnapshotReporter$new(snap_dir = "_snaps")
   withr::local_options(testthat.snapshotter = snapshotter)
   reporters <- list(
     SubprocessReporter$new(),
     snapshotter
   )
   multi <- MultiReporter$new(reporters = reporters)
-  with_reporter(multi, test_one_file(path, env = the$testing_env))
+  with_reporter(
+    multi,
+    test_one_file(path, env = the$testing_env, shuffle = shuffle)
+  )
   NULL
 }
 
@@ -303,45 +313,88 @@ queue_teardown <- function(queue) {
   tasks <- queue$list_tasks()
   num <- nrow(tasks)
 
+  # calling quit() here creates a race condition, and the output of
+  # the deferred_run() might be lost. Instead we close the input
+  # connection in a separate task.
   clean_fn <- function() {
     withr::deferred_run(.GlobalEnv)
-    quit(save = "no", status = 1L, runLast = TRUE)
   }
 
-  topoll <- list()
+  topoll <- integer()
   for (i in seq_len(num)) {
-    if (!is.null(tasks$worker[[i]])) {
+    if (
+      !is.null(tasks$worker[[i]]) && tasks$worker[[i]]$get_state() == "idle"
+    ) {
       # The worker might have crashed or exited, so this might fail.
       # If it does then we'll just ignore that worker
-      tryCatch({
-        tasks$worker[[i]]$call(clean_fn)
-        topoll <- c(topoll, tasks$worker[[i]]$get_poll_connection())
-      }, error = function(e) tasks$worker[i] <- list(NULL))
+      tryCatch(
+        {
+          tasks$worker[[i]]$call(clean_fn)
+          topoll <- c(topoll, i)
+        },
+        error = function(e) NULL
+      )
     }
   }
 
-  # Give covr time to write out the coverage files
-  if (in_covr()) grace <- 30L else grace <- 3L
+  # Give covr a bit more time
+  if (in_covr()) {
+    grace <- 30L
+  } else {
+    grace <- 1L
+  }
+  first_error <- NULL
   limit <- Sys.time() + grace
   while (length(topoll) > 0 && (timeout <- limit - Sys.time()) > 0) {
     timeout <- as.double(timeout, units = "secs") * 1000
-    pr <- processx::poll(topoll, as.integer(timeout))
+    conns <- lapply(tasks$worker[topoll], function(x) x$get_poll_connection())
+    pr <- unlist(processx::poll(conns, as.integer(timeout)))
+    for (i in which(pr == "ready")) {
+      msg <- tasks$worker[[topoll[i]]]$read()
+      first_error <- first_error %||% msg$error
+    }
+    topoll <- topoll[pr != "ready"]
+  }
+
+  topoll <- integer()
+  for (i in seq_len(num)) {
+    if (
+      !is.null(tasks$worker[[i]]) && tasks$worker[[i]]$get_state() == "idle"
+    ) {
+      tryCatch(
+        {
+          close(tasks$worker[[i]]$get_input_connection())
+          topoll <- c(topoll, i)
+        },
+        error = function(e) NULL
+      )
+    }
+  }
+
+  limit <- Sys.time() + grace
+  while (length(topoll) > 0 && (timeout <- limit - Sys.time()) > 0) {
+    timeout <- as.double(timeout, units = "secs") * 1000
+    conns <- lapply(tasks$worker[topoll], function(x) x$get_poll_connection())
+    pr <- unlist(processx::poll(conns, as.integer(timeout)))
     topoll <- topoll[pr != "ready"]
   }
 
   for (i in seq_len(num)) {
     if (!is.null(tasks$worker[[i]])) {
-      tryCatch(
-        close(tasks$worker[[i]]$get_input_connection()),
-        error = function(e) NULL
-      )
       if (ps::ps_is_supported()) {
-        tasks$worker[[i]]$kill_tree()
+        tryCatch(tasks$worker[[i]]$kill_tree(), error = function(e) NULL)
       } else {
-        tasks$worker[[i]]$kill()
+        tryCatch(tasks$worker[[i]]$kill(), error = function(e) NULL)
       }
     }
   }
+
+  if (!is.null(first_error)) {
+    cli::cli_abort(
+      "At least one parallel worker failed to run teardown",
+      parent = first_error
+    )
+  }
 }
 
 # Reporter that just forwards events in the subprocess back to the main process
@@ -351,7 +404,8 @@ queue_teardown <- function(queue) {
 # collect several of them and only emit a condition a couple of times
 # a second. End-of-test and end-of-file events would be transmitted
 # immediately.
-SubprocessReporter <- R6::R6Class("SubprocessReporter",
+SubprocessReporter <- R6::R6Class(
+  "SubprocessReporter",
   inherit = Reporter,
   public = list(
     start_file = function(filename) {
