@@ -1,14 +1,14 @@
 #' Manage test reporting
 #'
 #' The job of a reporter is to aggregate the results from files, tests, and
-#' expectations and display them in an informative way. Every testtthat function
+#' expectations and display them in an informative way. Every testthat function
 #' that runs multiple tests provides a `reporter` argument which you can
 #' use to override the default (which is selected by [default_reporter()]).
 #'
 #' You only need to use this `Reporter` object directly if you are creating
 #' a new reporter. Currently, creating new Reporters is undocumented,
 #' so if you want to create your own, you'll need to make sure that you're
-#' familiar with [R6](https://adv-r.hadley.nz/R6.html) and then need read the
+#' familiar with [R6](https://adv-r.hadley.nz/r6.html) and then need read the
 #' source code for a few.
 #'
 #' @keywords internal
@@ -23,20 +23,21 @@
 #' test_file(path)
 #' # Override the default by supplying the name of a reporter
 #' test_file(path, reporter = "minimal")
-Reporter <- R6::R6Class("Reporter",
+Reporter <- R6::R6Class(
+  "Reporter",
   public = list(
     capabilities = list(parallel_support = FALSE, parallel_updates = FALSE),
     start_reporter = function() {},
-    start_context =  function(context) {},
-    start_test =     function(context, test) {},
-    start_file =     function(filename) {},
-    add_result =     function(context, test, result) {},
-    end_test =       function(context, test) {},
-    end_context =    function(context) {},
-    end_reporter =   function() {},
-    end_file =       function() {},
-    is_full =        function() FALSE,
-    update =         function() {},
+    start_context = function(context) {},
+    start_test = function(context, test) {},
+    start_file = function(filename) {},
+    add_result = function(context, test, result) {},
+    end_test = function(context, test) {},
+    end_context = function(context) {},
+    end_reporter = function() {},
+    end_file = function() {},
+    is_full = function() FALSE,
+    update = function() {},
 
     width = 80,
     unicode = TRUE,
