@@ -1,5 +1,5 @@
-
-SnapshotReporter <- R6::R6Class("SnapshotReporter",
+SnapshotReporter <- R6::R6Class(
+  "SnapshotReporter",
   inherit = Reporter,
   public = list(
     snap_dir = character(),
@@ -7,21 +7,29 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
     test = NULL,
     test_file_seen = character(),
     snap_file_seen = character(),
+    snap_file_saved = character(),
     variants_changed = FALSE,
-    fail_on_new = FALSE,
+    fail_on_new = NULL,
+    desc = NULL,
 
     old_snaps = NULL,
     cur_snaps = NULL,
     new_snaps = NULL,
 
-    initialize = function(snap_dir = "_snaps", fail_on_new = FALSE) {
+    initialize = function(
+      snap_dir = "_snaps",
+      fail_on_new = NULL,
+      desc = NULL
+    ) {
       self$snap_dir <- normalizePath(snap_dir, mustWork = FALSE)
       self$fail_on_new <- fail_on_new
+      self$desc <- desc
     },
 
     start_file = function(path, test = NULL) {
       self$file <- context_name(path)
       self$test_file_seen <- c(self$test_file_seen, self$file)
+      self$snap_file_saved <- character()
 
       self$variants_changed <- character()
 
@@ -29,23 +37,45 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
       self$cur_snaps <- FileSnaps$new(self$snap_dir, self$file, type = "cur")
       self$new_snaps <- FileSnaps$new(self$snap_dir, self$file, type = "new")
 
+      if (!is.null(self$desc)) {
+        # When filtering tests, we need to copy over all of the old snapshots,
+        # apart from the one that matches the test
+        snaps <- self$old_snaps$snaps
+        test_name <- test_description(self$desc)
+        for (variant in names(snaps)) {
+          # In the case of subtests, snaps are named a / b / c1, a / b / c2 etc.
+          # So if we run a / b, we want to remove a / b, a / b / c, a / b / c2
+          # Subtests that use / in their names are not currently supported.
+          matches <- startsWith(names(snaps[[variant]]), test_name)
+          # Can't just remove because we want to preserve order
+          snaps[[variant]][matches] <- rep(list(NULL), sum(matches))
+        }
+        self$cur_snaps$snaps <- snaps
+      }
+
       if (!is.null(test)) {
         self$start_test(NULL, test)
       }
     },
 
     start_test = function(context, test) {
+      if (is.null(test)) {
+        return()
+      }
+
       self$test <- gsub("\n", "", test)
     },
 
     # Called by expectation
-    take_snapshot = function(value,
-                             save = identity,
-                             load = identity,
-                             ...,
-                             tolerance = testthat_tolerance(),
-                             variant = NULL,
-                             trace_env = NULL) {
+    take_snapshot = function(
+      value,
+      save = identity,
+      load = identity,
+      ...,
+      tolerance = testthat_tolerance(),
+      variant = NULL,
+      trace_env = caller_env()
+    ) {
       check_string(self$test, allow_empty = FALSE)
       i <- self$new_snaps$append(self$test, variant, save(value))
 
@@ -55,8 +85,10 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
         old <- load(old_raw)
 
         comp <- waldo_compare(
-          x = old,   x_arg = "old",
-          y = value, y_arg = "new",
+          x = old,
+          x_arg = "old",
+          y = value,
+          y_arg = "new",
           ...,
           tolerance = tolerance,
           quote_strings = FALSE
@@ -74,33 +106,44 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
         value_enc <- save(value)
 
         self$cur_snaps$append(self$test, variant, value_enc)
+        fail_on_new <- self$fail_on_new %||% on_ci()
 
         message <- paste0(
           "Adding new snapshot",
           if (variant != "_default") paste0(" for variant '", variant, "'"),
-          if (self$fail_on_new) " in CI",
           ":\n",
           value_enc
         )
-        if (self$fail_on_new) {
+        if (fail_on_new) {
           fail(message, trace_env = trace_env)
-        } else {
-          testthat_warn(message)
+          return()
         }
+        testthat_warn(message)
         character()
       }
     },
 
-    take_file_snapshot = function(name, path, file_equal, variant = NULL, trace_env = NULL) {
+    take_file_snapshot = function(
+      name,
+      path,
+      file_equal,
+      variant = NULL,
+      trace_env = caller_env()
+    ) {
       self$announce_file_snapshot(name)
 
-      if (is.null(variant)) {
-        snap_dir <- file.path(self$snap_dir, self$file)
-      } else {
-        snap_dir <- file.path(self$snap_dir, variant, self$file)
+      save_path <- paste0(c(self$file, variant, name), collapse = "/")
+      if (save_path %in% self$snap_file_saved) {
+        cli::cli_abort(
+          "Snapshot file names must be unique. {.arg name} has already been used.",
+          call = trace_env
+        )
       }
+      self$snap_file_saved <- c(self$snap_file_saved, save_path)
+
       snapshot_file_equal(
-        snap_test_dir = snap_dir,
+        snap_dir = self$snap_dir,
+        snap_test = self$file,
         snap_name = name,
         snap_variant = variant,
         path = path,
@@ -120,7 +163,11 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
         return()
       }
 
-      # If expectation errors or skips, need to reset remaining snapshots
+      # If expectation errors or skips, need to copy snapshots from old to cur
+      # TODO: the logic is not correct here for subtests, probably because
+      # the code was not written under the assumption that start_test()
+      # generates a stack of tests. You can see the problem by running
+      # local_on_cran() then testing describe.R.
       if (expectation_error(result) || expectation_skip(result)) {
         self$cur_snaps$reset(self$test, self$old_snaps)
       }
@@ -143,7 +190,8 @@ SnapshotReporter <- R6::R6Class("SnapshotReporter",
       # clean up if we've seen all files
       tests <- context_name(find_test_scripts(".", full.names = FALSE))
       if (!on_ci() && all(tests %in% self$test_file_seen)) {
-        snapshot_cleanup(self$snap_dir,
+        snapshot_cleanup(
+          self$snap_dir,
           test_files_seen = self$test_file_seen,
           snap_files_seen = self$snap_file_seen
         )
@@ -182,16 +230,34 @@ get_snapshotter <- function() {
 #'
 #' @export
 #' @keywords internal
-local_snapshotter <- function(snap_dir = NULL, cleanup = FALSE, fail_on_new = FALSE, .env = parent.frame()) {
-  snap_dir <- snap_dir %||% withr::local_tempdir(.local_envir = .env)
-  reporter <- SnapshotReporter$new(snap_dir = snap_dir, fail_on_new = fail_on_new)
-  if (!identical(cleanup, FALSE)) {
-    warn("`cleanup` is deprecated")
-  }
-
-  withr::local_options(
-    "testthat.snapshotter" = reporter,
-    .local_envir = .env
+local_snapshotter <- function(
+  reporter = SnapshotReporter,
+  snap_dir = "_snaps",
+  cleanup = FALSE,
+  desc = NULL,
+  fail_on_new = NULL,
+  frame = caller_env()
+) {
+  reporter <- reporter$new(
+    snap_dir = snap_dir,
+    fail_on_new = fail_on_new,
+    desc = desc
   )
+  withr::local_options("testthat.snapshotter" = reporter, .local_envir = frame)
+
   reporter
 }
+
+local_test_snapshotter <- function(
+  snap_dir = NULL,
+  desc = NULL,
+  frame = caller_env()
+) {
+  snap_dir <- snap_dir %||% withr::local_tempdir(.local_envir = frame)
+  local_snapshotter(
+    snap_dir = snap_dir,
+    desc = desc,
+    fail_on_new = FALSE,
+    frame = frame
+  )
+}
