@@ -4,7 +4,7 @@ local_edition(2)
 
 test_that("uses specified width", {
   old <- options(width = 20)
-  on.exit(options(old), add = TRUE)
+  withr::defer(options(old))
 
   x <- 1:100
   expect_known_output(print(x), "width-80.txt")
@@ -12,12 +12,9 @@ test_that("uses specified width", {
 
 test_that("creates file on first run", {
   file <- withr::local_tempfile()
-  expect_success(
-    expect_warning(
-      expect_known_output(cat("ok!\n"), file),
-      "Creating reference"
-    )
-  )
+  expect_known_output(cat("ok!\n"), file) |>
+    expect_success() |>
+    expect_warning("Creating reference")
 
   expect_true(file.exists(file))
 })
@@ -54,7 +51,7 @@ test_that("Warning for non-UTF-8 reference files", {
   Encoding(x) <- "latin1"
 
   tmp <- tempfile()
-  on.exit(unlink(tmp), add = TRUE)
+  withr::defer(unlink(tmp))
   writeBin(x, tmp)
 
   suppressWarnings(
@@ -67,7 +64,6 @@ test_that("Warning for non-UTF-8 reference files", {
 
 # expect_known_value ------------------------------------------------------
 
-
 test_that("correctly matches to a file", {
   x <- 1
   expect_success(expect_known_value(x, "one.rds"))
@@ -77,18 +73,15 @@ test_that("correctly matches to a file", {
 })
 
 test_that("first run is successful", {
-  expect_success(
-    expect_warning(
-      expect_known_value(2, "two.rds"),
-      "Creating reference"
-    )
-  )
+  expect_known_value(2, "two.rds") |>
+    expect_success() |>
+    expect_warning("Creating reference")
   unlink("two.rds")
 })
 
 test_that("equal_to_ref does not overwrite existing", {
-  tmp_rds <- tempfile(fileext=".rds")
-  on.exit(unlink(tmp_rds))
+  tmp_rds <- tempfile(fileext = ".rds")
+  withr::defer(unlink(tmp_rds))
   ref_obj1 <- 1:3
   ref_obj2 <- 2:4
   saveRDS(ref_obj1, tmp_rds)
@@ -100,14 +93,13 @@ test_that("equal_to_ref does not overwrite existing", {
   expect_equal(readRDS(tmp_rds), ref_obj1)
 
   # Now failure does update object
-  expect_failure(expect_equal_to_reference(ref_obj2, tmp_rds, update=TRUE))
+  expect_failure(expect_equal_to_reference(ref_obj2, tmp_rds, update = TRUE))
   expect_success(expect_equal_to_reference(ref_obj2, tmp_rds))
 })
 
 test_that("serializes to version 2 by default", {
-  skip_if(getRversion() < "3.5")
   tmp_rds <- tempfile(fileext = ".rds")
-  on.exit(unlink(tmp_rds))
+  withr::defer(unlink(tmp_rds))
 
   expect_warning(
     expect_known_value("a", tmp_rds),
@@ -118,9 +110,8 @@ test_that("serializes to version 2 by default", {
 })
 
 test_that("version 3 is possible", {
-  skip_if(getRversion() < "3.5")
   tmp_rds <- tempfile(fileext = ".rds")
-  on.exit(unlink(tmp_rds))
+  withr::defer(unlink(tmp_rds))
 
   expect_warning(
     expect_known_value("a", tmp_rds, version = 3),
@@ -133,12 +124,9 @@ test_that("version 3 is possible", {
 # expect_known_hash -------------------------------------------------------
 
 test_that("empty hash succeeds with warning", {
-  expect_success(
-    expect_warning(
-      expect_known_hash(1:10),
-      "No recorded hash"
-    )
-  )
+  expect_known_hash(1:10) |>
+    expect_success() |>
+    expect_warning("No recorded hash")
 })
 
 test_that("only succeeds if hash is correct", {
