@@ -3,26 +3,37 @@ test_that("expectation contains failure message even when successful", {
   expect_equal(e$message, "I failed")
 })
 
-test_that("expect warns if no `failure_message`", {
-  expect_warning(expect(TRUE), "missing, with no default")
+test_that("info only evaluated on failure", {
+  expect_no_error(expect(TRUE, "fail", info = stop("!")))
 })
 
-test_that("info only evaluated on failure", {
-  expect_error(expect(TRUE, "fail", info = stop("!")), NA)
+test_that("validates key inputs", {
+  expect_snapshot(error = TRUE, {
+    expect(1)
+    expect(TRUE, 1)
+  })
 })
 
 test_that("can subclass expectation", {
-  exp <- new_expectation("failure", "didn't work", .subclass = "foo", bar = "baz")
-  expect_true(inherits_all(exp, c("foo", "expectation_failure", "expectation", "error", "condition")))
+  exp <- new_expectation(
+    "failure",
+    "didn't work",
+    .subclass = "foo",
+    bar = "baz"
+  )
+  expect_true(inherits_all(
+    exp,
+    c("foo", "expectation_failure", "expectation", "error", "condition")
+  ))
   expect_identical(attr(exp, "bar"), "baz")
 })
 
 test_that("`expect()` and `exp_signal()` signal expectations", {
-  expect_error(expect(TRUE, ""), regexp = NA)
-  expect_error(expect(FALSE, ""), class = "expectation_failure")
+  expect_success(expect(TRUE, ""))
+  expect_failure(expect(FALSE, ""))
 
-  expect_error(exp_signal(new_expectation("success", "")), regexp = NA)
-  expect_error(exp_signal(new_expectation("failure", "")), class = "expectation_failure")
+  expect_success(exp_signal(new_expectation("success", "")))
+  expect_failure(exp_signal(new_expectation("failure", "")))
 })
 
 test_that("conditionMessage() is called during conversion", {
@@ -35,8 +46,14 @@ test_that("conditionMessage() is called during conversion", {
   expect_identical(as.expectation(wrn)$message, "dispatched")
 
   err <- error_cnd("foobar", message = "wrong")
-  expect_identical(as.expectation(err)$message, "dispatched")
+  expect_match(as.expectation(err)$message, "Error: dispatched")
 
   err <- cnd(c("foobar", "skip"), message = "wrong")
   expect_identical(as.expectation(err)$message, "dispatched")
 })
+
+test_that("error message includes call", {
+  f <- function() stop("Error!")
+  cnd <- catch_cnd(f())
+  expect_equal(format(as.expectation(cnd)), "Error in `f()`: Error!")
+})
