@@ -6,12 +6,19 @@ test_that("crash", {
   withr::local_envvar(TESTTHAT_PARALLEL = "TRUE")
 
   pkg <- test_path("test-parallel", "crash")
-  err <- callr::r(function() {
-    tryCatch(
-      testthat::test_local(".", reporter = "summary", stop_on_failure = FALSE),
-      error = function(e) e
-    )
-  }, wd = pkg)
+  err <- callr::r(
+    function() {
+      tryCatch(
+        testthat::test_local(
+          ".",
+          reporter = "summary",
+          stop_on_failure = FALSE
+        ),
+        error = function(e) e
+      )
+    },
+    wd = pkg
+  )
   expect_s3_class(err, "testthat_process_error")
   expect_equal(err$test_file, "test-crash-3.R")
 })
