@@ -1,8 +1,4 @@
 test_that("captures error before first test", {
-  # Backtrace srcrefs failure
-  skip_on_covr()
-  skip_if(is.null(attr(rlang::eval_bare, "srcref")))
-
   local_output_override()
 
   expect_snapshot_reporter(
@@ -47,10 +43,7 @@ test_that("can fully suppress incremental updates", {
 })
 
 test_that("reports backtraces", {
-  # Avoid failures because of different srcrefs in backtraces
-  skip_on_covr()
-  skip_if(is.null(attr(rlang::eval_bare, "srcref")))
-
+  withr::local_envvar(TESTTHAT_MAX_FAILS = Inf)
   expect_snapshot_reporter(
     ProgressReporter$new(update_interval = 0, min_time = Inf),
     test_path("reporters/backtraces.R")
@@ -79,9 +72,25 @@ test_that("display of successes only is compact", {
     test_path("reporters/successes.R")
   )
 
-  # And even more compact if in RStudio pane
   expect_snapshot_reporter(
-    CompactProgressReporter$new(rstudio = TRUE),
+    CompactProgressReporter$new(),
+    test_path("reporters/skips.R")
+  )
+
+  # And even more compact if in RStudio pane
+  local_reproducible_output(rstudio = TRUE)
+  expect_snapshot_reporter(
+    CompactProgressReporter$new(),
     test_path("reporters/successes.R")
   )
 })
+
+# parallel progress reporter ----------------------------------------------
+
+test_that("ParallelProgressReporter fails after max_fail tests", {
+  withr::local_options(testthat.progress.max_fails = 10)
+  expect_snapshot_reporter(
+    ParallelProgressReporter$new(update_interval = 0, min_time = Inf),
+    test_path(c("reporters/fail-many.R", "reporters/fail.R"))
+  )
+})
